import { PromptRunner, PromptNode } from 'prompt-anything';
import { DiscordChannel } from './DiscordChannel';
import { TextChannel, User, Message } from 'discord.js';
/**
 * Runs a series of prompt nodes
 */
export declare class DiscordPromptRunner<DataType> extends PromptRunner<DataType, Message> {
    /**
     * Channel IDs that are currently running prompts
     */
    static activeChannels: Set<string>;
    /**
     * @param author User who started the prompt
     * @param data Initial data
     */
    constructor(author: User, data: DataType);
    /**
     * Marks a channel as active
     *
     * @param id Channel ID
     */
    static addActiveChannel(id: string): void;
    /**
     * Unmarks a channel as active
     *
     * @param id Channel ID
     */
    static deleteActiveChannel(id: string): void;
    /**
     * Check if a channel is currently running a prompt. This
     * should be called before running a prompt runner to
     * check if there are any active menus before running
     * a new one
     *
     * @param id Channel ID
     */
    static isActiveChannel(id: string): boolean;
    /**
     * Convert a TextChannel to a DiscordChannel for
     * compatibility with prompt-anything
     *
     * @param channel
     */
    static convertTextChannel(channel: TextChannel): DiscordChannel;
    run(node: PromptNode<DataType, Message>, channel: DiscordChannel): Promise<DataType>;
    run(node: PromptNode<DataType, Message>, channel: TextChannel): Promise<DataType>;
}
